<template>
  <Layout>
    <div class="relative block items-center justify-center bg-white rounded-xl overflow-hidden shadow-xl h-full">
      <CreditLimit
          v-if="dashboardStore.vendorCredit.isLimit == true || dashboardStore.vendorSubscribe.isExpired == true"/>
      <template v-else>
        <ChatNotSelected v-if="!chat.selectedChat"/>
        <ChatData ref="chatData" v-else/>
      </template>
    </div>
  </Layout>
</template>
<script setup lang="ts">
import {SchemaSyncHandler} from './utils/SchemaSyncHandler';
import Layout from './layouts/withHeader.vue'
import ChatData from "./components/Section/ChatData.vue";
import ChatNotSelected from "./components/Error/ChatNotSelected.vue";
import {useChatListStore} from "./stores/chat_list";
import {onBeforeMount, onMounted, ref, watch} from "vue";
import {getCurrentInstance} from 'vue'
import firebaseMessaging from './plugins/firebase'
import { payload_socket, waitForSocketConnection } from './plugins/socket';
import {onMessage, getToken} from "firebase/messaging";
import {useGeneralStore} from "./stores/general";
import helpers from "./utils/helpers";
import {useUserStore} from "./stores/user";
import {useDashboardStore} from "./stores/dashboard";
import CreditLimit from "./components/Error/CreditLimit.vue";

const token = ref()
const chatData = ref()
const chat = useChatListStore()
const useUser = useUserStore()
const generalStore = useGeneralStore()
const dashboardStore = useDashboardStore()

// const receiveMessage = async () => {
//   onMessage(firebaseMessaging, (payload) => {
//     console.log(payload, 'event frontground message')

//     handleFirebase(payload)
//   });
// }

// const handleFirebase = async (payload) => {
//   if (payload?.data?.type === 'auto_logout') {
//     generalStore.openDialog(payload?.data?.title, payload?.data?.body, true)
//   } else {
//     let is_same_tab = 0
//     let messageData = payload?.data?.message

//     if(helpers.isJsonString(messageData)) {
//       messageData = JSON.parse(payload?.data?.message)
//     }

//     if (generalStore.chatTab === 'whatsapp' && messageData?.phone_number_id != null) {
//       is_same_tab = 1
//     }

//     if (generalStore.chatTab === 'instagram' && (messageData?.instagram_id != null || messageData?.page_id != null)) {
//       is_same_tab = 1
//     }

//     if (generalStore.chatTab === 'email' && messageData?.email != null) {
//       is_same_tab = 1
//     }
    
//     if (generalStore.chatTab === 'google' && messageData?.brand_id != null) {
//       is_same_tab = 1
//     }

//     if(is_same_tab == 1 && (chat.tabName)?.toLowerCase() === (messageData.session)?.toLowerCase()) {
      
//       if (generalStore.chatTab === 'whatsapp') {
//         useUser.fetchUserProfile()
//       }

//       if(chat.isListFiltered == false && chat.searchString.length == 0) {
//         appendExisitingChatList(payload?.data?.user_chat_id, payload)
//       }
//     }

//     //handle chat pindah tab
//     if(is_same_tab == 1 && (chat.tabName)?.toLowerCase() != (messageData.session)?.toLowerCase()) {
      
//       let before_message = payload?.data?.message
//       if(helpers.isJsonString(before_message)) {
//         before_message = JSON.parse(payload?.data?.message)
//       }
      
//       const payload_message = before_message
//       chat.list.map((chat_list, index) => {
//         if(chat_list.user_chat_id === parseInt(payload_message.user_chat_id)){
//           chat.list.splice(index, 1)
//         }
//       });
//     }
//     //handle chat pindah tab

//     if (chat.selectedChat === parseInt(payload.data.user_chat_id)) {
//       let isUpdateChat = false

//       //handle read receipt message
//       if (chat.messages.length > 0 && payload?.data?.message) {
        
//         let before_message = payload?.data?.message
//         if(helpers.isJsonString(before_message)) {
//           before_message = JSON.parse(payload?.data?.message)
//         }
        
//         const payload_message = before_message
//         chat.messages.map((chating, index) => {
//             if(chating?.id === parseInt(payload_message.chat_id)){
//               //check if status read, make all messages status read
//               if(payload_message.last_message_status == 'read'){
//                 chat.messages.map((c, i) => {
//                   c.last_message_status = payload_message.last_message_status
//                   c.status = payload_message.last_message_status
//                 })
//               // check if status read, make all messages status read
//               }else{
//                 chat.messages[index].last_message_status = helpers.messageStatus(chat.messages[index].last_message_status, payload_message.last_message_status)
//                 chat.messages[index].status = helpers.messageStatus(chat.messages[index].status, payload_message.last_message_status)
                
//                 // chat.messages[index].last_message_status = payload_message.last_message_status
//                 // chat.messages[index].status = payload_message.last_message_status
//               }

//               isUpdateChat = true
//             }
//         });
//       }
//       //handle read receipt message

//       if(isUpdateChat == false){
//         chatData.value.fetchChatDetail(payload.data.user_chat_id, false)
//       }
//     }
    
//     generalStore.getQueueChat()
//     // chat.getUnreadCount()
//   }
  
// }

const handleStatusAgent = async (payload) => {
  let dataPayload = payload?.data

  if(helpers.isJsonString(dataPayload)) {
    dataPayload = JSON.parse(payload?.data)
  }

  let index_agent = chat.listAgent.map(item => item.id).indexOf(dataPayload.id)
  if (chat.listAgent[index_agent]) {
    chat.listAgent[index_agent].is_online = dataPayload.is_online
  }

  if(dataPayload.id == useUser.user.id) {
    useUser.user.is_online = dataPayload.is_online
  }
}

const handleMessageReceived = async (payload) => {
  let is_same_tab = 0

  let messageData = payload?.data?.message
  if(helpers.isJsonString(messageData)) {
    messageData = JSON.parse(payload?.data?.message)
  }

  if (generalStore.chatTab === 'whatsapp' && messageData?.phone_number_id != null) {
    is_same_tab = 1
  }

  if (generalStore.chatTab === 'instagram' && (messageData?.instagram_id != null || messageData?.page_id != null)) {
    is_same_tab = 1
  }

  if (generalStore.chatTab === 'email' && messageData?.email != null) {
    is_same_tab = 1
  }
  
  if (generalStore.chatTab === 'google' && messageData?.brand_id != null) {
    is_same_tab = 1
  }

  if (generalStore.chatTab === 'tiktok' && messageData?.tiktok_id != null) {
    is_same_tab = 1
  }

  if(messageData?.tiktok_id){
    if(!useUser.listAccess['access_view_chat_tiktok']){
      return
    }
  }
  if(messageData?.email){
    if(!useUser.listAccess['access_view_chat_email']){
      return
    }
  }
  
  if(messageData?.phone_number_id){
    if(!useUser.listAccess['access_view_chat_whatsapp']){
      return
    }
  }

  if(messageData?.page_id != null || messageData?.instagram_id != null){
    if(!useUser.listAccess['access_view_chat_instagram']){
      return
    }
  }

  if(is_same_tab == 1 && (chat.tabName)?.toLowerCase() === (messageData.session)?.toLowerCase()) {
    
    if (generalStore.chatTab === 'whatsapp') {
      useUser.fetchUserProfile()
      chat.getUnreadCount()
    }

    if (chat.isListFiltered || chat.searchString.length > 0) return;
    if (useUser.user?.is_agent === 1 && !useUser.user?.is_online) return;
    if (messageData?.warehouse_id && messageData?.warehouse_id !== useUser.user?.warehouse_id)  return;

    appendChatList(payload?.data?.user_chat_id, payload);
  }

  //handle chat pindah tab
  if(is_same_tab == 1 && (chat.tabName)?.toLowerCase() != (messageData.session)?.toLowerCase()) {
    
    let before_message = payload?.data?.message
    if(helpers.isJsonString(before_message)) {
      before_message = JSON.parse(payload?.data?.message)
    }
    
    const payload_message = before_message
    chat.list.map((chat_list, index) => {
      if(chat_list.user_chat_id === parseInt(payload_message.user_chat_id)){
        chat.list = chat.list.filter((_, i) => i !== index)
        chat.unreadCount.need_fu--
      }
    })
  }
  //handle chat pindah tab

  //handle agent pindah assign
  if(is_same_tab == 1 && useUser.user?.is_agent == 1) {
    
    let before_message = payload?.data?.message
    if(helpers.isJsonString(before_message)) {
      before_message = JSON.parse(payload?.data?.message)
    }
    
    const payload_message = before_message
    chat.list.map((chat_list, index) => {
      if(chat_list.user_chat_id === parseInt(payload_message.user_chat_id) && useUser?.user?.id != parseInt(payload_message.user_vendor_id) && !payload?.data?.is_in_group){
        chat.list.splice(index, 1)
      }
    })

    if(payload_message.user_chat_id == chat.selectedChat && useUser?.user?.id != parseInt(payload_message.user_vendor_id) && !payload?.data?.is_in_group){  
      chat.selectedChat = null
    }
  }
  //handle agent pindah assign

  //handle background notification windows
  if (document.hidden && Notification.permission === 'granted' && (messageData.session)?.toLowerCase() == 'need_fu') {
    if(useUser.user?.is_agent == 1 && useUser?.user?.id != parseInt(messageData.user_vendor_id) && !payload?.data?.is_in_group){
      return
    }

    let body = payload?.data?.body || '';
    let title = payload?.data?.title || '';

    let output = `${title + ':'}\n`;

    if (body.length > 30) {
        body = body.substring(0, 30) + '...';
    }

    output += body;


    var notify = new Notification('Chat Desktop Halosis', {
      body: output,
      icon: window.location.origin+'/halosis.svg',
    })
  }
  //handle background notification windows

  //handle append chat in chat room
  if(generalStore.chatTab !== 'email'){
    if (chat.selectedChat === parseInt(payload.data.user_chat_id)) {
      chatData.value.fetchChatDetail(payload.data.user_chat_id, false)
    }
  }
  else{
    let chatid = helpers.encodeChatId(payload.data.user_chat_id, payload.data?.email_chat_session_id);

    if (chat.selectedChat === chatid) {
      chatData.value.fetchChatDetail(chatid, true)
    }
  }
  //handle append chat in chat room
  
  generalStore.getQueueChat()
  // generalStore.getQueueChatFromSocket(payload.data.count)
  // chat.getUnreadCount()
}

const handleReadReceipt = async (payload) => {
  let is_same_tab = 0

  let messageData = payload?.data?.message
  if(helpers.isJsonString(messageData)) {
    messageData = JSON.parse(payload?.data?.message)
  }

  if (generalStore.chatTab === 'whatsapp' && messageData?.phone_number_id != null) {
    is_same_tab = 1
  }

  if (generalStore.chatTab === 'instagram' && (messageData?.instagram_id != null || messageData?.page_id != null)) {
    is_same_tab = 1
  }

  if (generalStore.chatTab === 'email' && messageData?.email != null) {
    is_same_tab = 1
  }
  
  if (generalStore.chatTab === 'google' && messageData?.brand_id != null) {
    is_same_tab = 1
  }

  if (generalStore.chatTab === 'tiktok' && messageData?.tiktok_id != null) {
    is_same_tab = 1
  }

  if(is_same_tab == 1 && (chat.tabName)?.toLowerCase() === (messageData.session)?.toLowerCase()) {
    //handle chat list receipt
    let index_chat = null

    chat.list.map((chat_list, index) => {
      if(chat_list.user_chat_id === parseInt(messageData.user_chat_id)){
        index_chat = index
      }
    })

    if(index_chat != null) {
      let incoming_last_message_time = messageData.last_message_time.replace(/-/g, "/")
      incoming_last_message_time = new Date(incoming_last_message_time)
      
      let item_chat_last_message_time = chat.list[index_chat].last_message_time.replace(/-/g, "/")
      let item_last_message_time = new Date(item_chat_last_message_time)

      if(incoming_last_message_time >= item_last_message_time){
        chat.list[index_chat].last_message_status = helpers.messageStatus(chat.list[index_chat].last_message_status, messageData.last_message_status)
      }
    }
    //handle chat list receipt
  }

  if (chat.selectedChat === parseInt(payload.data.user_chat_id)) {

    //handle read receipt message
    if (chat.messages.length > 0 && payload?.data?.message) {
      
      const payload_message = messageData

      chat.messages.map((chating, index) => {
          if(chating?.id === parseInt(payload_message.chat_id)){
            //check if status read, make all messages status read
            if(payload_message.last_message_status == 'read'){
              chat.messages.map((c, i) => {
                c.last_message_status = payload_message.last_message_status
                c.status = payload_message.last_message_status
              })
            // check if status read, make all messages status read
            }else{
              chat.messages[index].last_message_status = helpers.messageStatus(chat.messages[index].last_message_status, payload_message.last_message_status)
              chat.messages[index].status = helpers.messageStatus(chat.messages[index].status, payload_message.last_message_status)
            }
          }
      });
    }
    //handle read receipt message
  }
}

const handleAssignLabel = async (payload) => {
  let dataPayload = payload

  if(helpers.isJsonString(dataPayload)) {
    dataPayload = JSON.parse(payload)
  }

  generalStore.labels = dataPayload?.labels

  let chatList = chat.list.map(item => item.user_chat_id).indexOf(dataPayload.chat_user_id)
  if (chat.list[chatList]) {
    chat.list[chatList] = {
      ...chat.list[chatList],
      labels: dataPayload?.labels
    }
  }
  
  if(dataPayload.chat_user_id == chat.selectedChat) {
    await useUser.fetchCustomerProfile(chat.selectedChat)
  }
  
}

const generateNewToken = async () => {
  const instance = getCurrentInstance();
  const urlSearchParams = new URLSearchParams(window.location.search);
  const params = Object.fromEntries(urlSearchParams.entries());
  // const storageToken = localStorage.getItem('kv-auth') ?? null
  const deviceToken = localStorage.getItem('device_token') ?? null

  // if (!storageToken) {
  let vid_param = null
  let vid = null
  let is_bypass = 0

  if(params.vid != null){
    vid = helpers.decodeMessage(params.vid)
    vid_param = params.vid
    is_bypass = 1
  }
    
  if(params.pnb != null){
    let pnb = helpers.decodeMessage(params.pnb)
    chat.isFirstOpenChat = true
    chat.firstPhoneNumber = pnb
  }

  if(params.text_content != null){
    let first_text = helpers.decodeMessage(params.text_content)
    chat.firstText = first_text
  }

  const screenWidth = window.innerWidth;
  if (screenWidth < 500) {
    generalStore.isShowRedirectToMobile = true;
  }
  
  await generalStore.generateToken({
    token: params.token,
    is_desktop: 1,
    device_token: generalStore.deviceToken ?? deviceToken,
    is_bypass: is_bypass,
    vid: vid_param
  }).then(async (resp) => {
    if (resp?.data?.token) {
      instance?.proxy?.$forceUpdate();
      
      if(is_bypass && vid != resp.data.vendor_id){        
        generalStore.is_same_vendor = false
        console.log("vendor id tidak sama")
      }

      if (generalStore.deviceToken == null) {
        await getToken(firebaseMessaging, {vapidKey: "BH1cobW8UnpY1KRrbFsDaR9eciWDz3RTB9YVnn_wtMx3VfGzuUC7ZqR1qbFwQM6DFnmNZjh-l_YgKpopl7azdpo"})
        .then(async (currentToken) => {
          if (currentToken) {
            token.value = currentToken
            localStorage.setItem("device_token", currentToken)
            await generalStore.updateDeviceToken(currentToken)
          }
        })
        .catch((e) => console.log(e, 'error'))
      }
    } else {
      generalStore.token = localStorage.getItem("kv-auth")
    }
  })
  .catch((e) => {
    console.log(e, 'error get token')
  })
  // }
}

const setOffline = async () => {
  await useUser.setOnlineStatus(0)
}

const appendExisitingChatList = (chat_user_id, response: any) => {
  if (chat_user_id) {
    
    if(response?.data?.type == "chat_room") {
      return true
    }

    console.log("[1]", response?.data?.message)
    const oldChatList = chat.list

    let chatData = response?.data?.message
    chatData['is_blur'] = response?.data?.is_blur

    if(helpers.isJsonString(chatData)) {
      chatData = JSON.parse(response?.data?.message)
    }

    let isRemove = true
    
    if(chat.list.length > 0){
      let new_chat_date = chatData.last_message_time.replace(/-/g, "/")
      new_chat_date = new Date(new_chat_date)
      
      let last_chat_date = chat.list[0].last_message_time.replace(/-/g, "/")
      let last_chatdate = new Date(last_chat_date)

      if(new_chat_date < last_chatdate && useUser.user?.is_agent == 0){
        isRemove = false
      }
    }

    if (chatData?.is_expired) {

      chat.updateChatList(chatData, chat_user_id, true, false, isRemove)

    } else {
      console.log("[2]", chat_user_id)
      if (useUser.user?.is_agent == 1) {
        console.log("[3]", useUser.user?.is_agent)
        if (useUser?.user?.id === parseInt(chatData?.user_vendor_id)) {
          console.log("[4]", chatData)
          chat.updateChatList( chatData, chat_user_id)
          // chat.list = [
          //   chatData,
          //   ...oldChatList
          // ]
          if (response?.data?.chat_from === "from_customer") {
            try {
              helpers.notificationSound(chat.selectedChat ? 'innerchat' : 'global')
            } catch (e) {
            }
          }
        } else {

          console.log("[5]", chatData)
          if (chatData?.user_vendor_id !== null) {
            if (chatData?.user_vendor_id !== useUser?.user?.id) {
              console.log("[6]", chatData)
              chat.updateChatList(chatData, chat_user_id, false)
            }
          }
        }
      } else {
        console.log("[7.a]", response?.data?.message)
        console.log("[7]", chatData, chat_user_id)
        chat.updateChatList(chatData, chat_user_id, true, false, isRemove)
        // chat.list = [
        //   chatData,
        //   ...oldChatList
        // ]
        if (response?.data?.chat_from === "from_customer") {
          try {
            helpers.notificationSound(chat.selectedChat ? 'innerchat' : 'global')
          } catch (e) {
          }
        }
      }
    }
  }
}

const appendChatList = (chat_user_id, response: any) => {
  if (chat_user_id) {
    let chatData = response?.data?.message

    if(helpers.isJsonString(chatData)) {
      chatData = JSON.parse(response?.data?.message)
    }
    
    chatData['is_blur'] = response?.data?.is_blur

    if (useUser.user?.is_agent == 1) {
      if (useUser?.user?.id === parseInt(chatData?.user_vendor_id) || response?.data?.is_in_group) {
        chat.updateChatList( chatData, chat_user_id)
      }
    } else {
      if(generalStore.chatTab == 'email'){
        chat_user_id = helpers.encodeChatId(chat_user_id, response?.data?.email_chat_session_id)
        chatData['user_chat_id'] = chat_user_id
        chatData['email_subject'] = response?.data?.email_subject
        chatData['email_chat_session_id'] = response?.data?.email_chat_session_id
      }
      
      chat.updateChatList(chatData, chat_user_id, true, false)
    }

    
    // if (response?.data?.chat_from === "from_customer") {
    //   try {
    //     helpers.notificationSound(chat.selectedChat ? 'innerchat' : 'global')
    //   } catch (e) {
    //     console.log('error norification sound', e)
    //   }
    // }
  }
}

const handleBeforeUnload = async (event) => {
  //disconnect ws
  const payload_unsubscribe = payload_socket('unsubscribe')
  generalStore.web_socket.send(payload_unsubscribe)
  //disconnect ws

  localStorage.setItem("filter_label", JSON.stringify([]))
  localStorage.setItem("filter_agent", JSON.stringify([]))
}


const reconnect_socket = async () => {
  // if (confirm("Disconnect, want to reconnect?")) {
    connect_web_socket()
  // }
}


const connect_web_socket = async () => {
  generalStore.web_socket = new WebSocket('wss://'+import.meta.env.VITE_WS_HOST+'/app/'+import.meta.env.VITE_PUSHER_KEY+'?protocol=7&client=js&version=8.4.0-rc2&flash=false')

  const payload_subscribe = payload_socket('subscribe')
  const payload_unsubscribe = payload_socket('unsubscribe')
  const payload_heartbeat = payload_socket('ping')

  waitForSocketConnection(generalStore.web_socket, function(){
    generalStore.web_socket.send(payload_subscribe)

    generalStore.web_socket.onmessage = function(msg) {
      let resp_socket = JSON.parse(msg.data)
      if(resp_socket.event != "pusher:pong"){
        console.log('data mentahan socket', resp_socket)
      }

      const deviceToken = localStorage.getItem('device_token') ?? null
      let dev_t = generalStore.deviceToken ?? deviceToken

      
      //handle read receipt
      if(resp_socket.event == "message_status.event"){
        let data_socket = JSON.parse(resp_socket.data)
        console.log('data dari socket read receipt',data_socket)
        handleReadReceipt(data_socket.payload)
      }
      //handle read receipt

      
      //handle assign label
      if(resp_socket.event == "label.event"){
        let data_socket = JSON.parse(resp_socket.data)
        console.log('data dari socket assign label',data_socket)
        handleAssignLabel(data_socket.payload)
        // handleFirebase(data_receipt.payload)
      }
      //handle assign label

      //handle message received
      if(resp_socket.event == "message_received.event"){
        let data_socket = JSON.parse(resp_socket.data)
        console.log('data dari socket message received',data_socket)
        handleMessageReceived(data_socket.payload)
      }
      //handle message received

      //handle status agent
      if(resp_socket.event == "status_agent.event"){
        let data_socket = JSON.parse(resp_socket.data)
        console.log('data dari socket status agent',data_socket)
        handleStatusAgent(data_socket.payload)
      }
      //handle status agent

      //handle auto logout
      if(resp_socket.event == "auto_logout.event"){
        let data_socket = JSON.parse(resp_socket.data)
        console.log('data dari socket auto logout',data_socket)
        generalStore.openDialog(data_socket?.payload?.title, data_socket?.payload?.body, true)
      }
      //handle  logout

      //handle user permission ke update
      if(resp_socket.event == "user_permission.event"){
        generalStore.openDialog('Akses fitur chat Anda telah diperbarui', 'Silakan hubungi Administrator', false)
      }
      //handle user permission ke update
    }

    //handle ping pong
    generalStore.web_socket.timer = setInterval(function() { generalStore.web_socket.send(payload_heartbeat) }, 15000)
    //handle ping pong
    
    //handle socket mode
    generalStore.checkSocketMode().then((resp) => {
      if(resp?.code == 200 && resp?.data?.socket_mode == 1){
        
      }else{
        generalStore.web_socket.send(payload_unsubscribe)
      }
    })
    //handle socket mode
  })
  
  generalStore.web_socket.onclose = () => {
    reconnect_socket()
  }
}

onBeforeMount(async () => {
  window.addEventListener("beforeunload", handleBeforeUnload)
})

onMounted(async () => {
  await generateNewToken()

  useUser.fetchDataGroup()
  
  generalStore.fetchDataFilterTicket()
  generalStore.getMaximumPin()
  
  await useUser.fetchUserProfile()

  //handle background firebase
  // const channel = new window.BroadcastChannel("sw-messages")
  // channel.addEventListener('message', async (event) => {
  //   console.log(event, "event background firebase")
  //   handleFirebase(event.data)

  // })
  //handle background firebase
  sessionStorage.clear();
  await SchemaSyncHandler.clearStore('in_progress_messages')

  if(useUser.user?.id){
    connect_web_socket()
  }
})

watch(() => generalStore.token, async (val) => {
  if (val) {
    await generalStore.getDashboard()
    await dashboardStore.fetchVendorWaba()

    await useUser.fetchUserProfile()
    await useUser.fetchDataGroup()
  }

  // await receiveMessage()
})

// onBeforeMount(async () => {
//   await generalStore.getProfile()
// })

</script>

<style scoped lang="css">
.scrollbar-hide::-webkit-scrollbar {
  display: none;
}

/* For IE, Edge and Firefox */
.scrollbar-hide {
  -ms-overflow-style: none; /* IE and Edge */
  scrollbar-width: none; /* Firefox */
}
</style>
